@@ -8,7 +8,9 @@ use solana_sdk::{
     account::{AccountSharedData, ReadableAccount, WritableAccount},
     account_utils::StateMut,
     bpf_loader_upgradeable::{self, UpgradeableLoaderState},
-    feature_set::{demote_sysvar_write_locks, instructions_sysvar_enabled, FeatureSet},
+    feature_set::{
+        instructions_sysvar_enabled, neon_evm_compute_budget, updated_verify_policy, FeatureSet,
+    },
     ic_logger_msg, ic_msg,
     instruction::{CompiledInstruction, Instruction, InstructionError},
     keyed_account::{create_keyed_accounts_unified, keyed_account_at_index, KeyedAccount},
@@ -103,6 +105,7 @@ impl PreAccount {
         post: &AccountSharedData,
         timings: &mut ExecuteDetailsTimings,
         outermost_call: bool,
+        updated_verify_policy: bool,
     ) -> Result<(), InstructionError> {
         let pre = self.account.borrow();
 
@@ -171,9 +174,14 @@ impl PreAccount {
             if !rent.is_exempt(post.lamports(), post.data().len()) {
                 return Err(InstructionError::ExecutableAccountNotRentExempt);
             }
+            let owner = if updated_verify_policy {
+                post.owner()
+            } else {
+                pre.owner()
+            };
             if !is_writable // line coverage used to get branch coverage
                 || pre.executable()
-                || program_id != pre.owner()
+                || program_id != owner
             {
                 return Err(InstructionError::ExecutableModified);
             }
@@ -253,9 +261,8 @@ impl ComputeMeter for ThisComputeMeter {
 pub struct ThisInvokeContext<'a> {
     invoke_stack: Vec<InvokeContextStackFrame<'a>>,
     rent: Rent,
-    message: &'a Message,
     pre_accounts: Vec<PreAccount>,
-    account_deps: &'a [(Pubkey, Rc<RefCell<AccountSharedData>>)],
+    accounts: &'a [(Pubkey, Rc<RefCell<AccountSharedData>>)],
     programs: &'a [(Pubkey, ProcessInstructionWithContext)],
     logger: Rc<RefCell<dyn Logger>>,
     bpf_compute_budget: BpfComputeBudget,
@@ -277,8 +284,7 @@ impl<'a> ThisInvokeContext<'a> {
         message: &'a Message,
         instruction: &'a CompiledInstruction,
         executable_accounts: &'a [(Pubkey, Rc<RefCell<AccountSharedData>>)],
-        accounts: &'a [Rc<RefCell<AccountSharedData>>],
-        account_deps: &'a [(Pubkey, Rc<RefCell<AccountSharedData>>)],
+        accounts: &'a [(Pubkey, Rc<RefCell<AccountSharedData>>)],
         programs: &'a [(Pubkey, ProcessInstructionWithContext)],
         log_collector: Option<Rc<LogCollector>>,
         bpf_compute_budget: BpfComputeBudget,
@@ -294,14 +300,12 @@ impl<'a> ThisInvokeContext<'a> {
             instruction,
             executable_accounts,
             accounts,
-            feature_set.is_active(&demote_sysvar_write_locks::id()),
         );
         let mut invoke_context = Self {
             invoke_stack: Vec::with_capacity(bpf_compute_budget.max_invoke_depth),
             rent,
-            message,
             pre_accounts,
-            account_deps,
+            accounts,
             programs,
             logger: Rc::new(RefCell::new(ThisLogger { log_collector })),
             bpf_compute_budget,
@@ -354,25 +358,21 @@ impl<'a> InvokeContext for ThisInvokeContext<'a> {
         let keyed_accounts = keyed_accounts
             .iter()
             .map(|(is_signer, is_writable, search_key, account)| {
-                self.account_deps
+                self.accounts
                     .iter()
-                    .map(|(key, _account)| key)
-                    .chain(self.message.account_keys.iter())
-                    .position(|key| key == *search_key)
-                    .map(|mut index| {
+                    .position(|(key, _account)| key == *search_key)
+                    .map(|index| {
                         // TODO
                         // Currently we are constructing new accounts on the stack
                         // before calling MessageProcessor::process_cross_program_instruction
                         // Ideally we would recycle the existing accounts here.
-                        let key = if index < self.account_deps.len() {
-                            &self.account_deps[index].0
-                            // &self.account_deps[index].1 as &RefCell<AccountSharedData>,
-                        } else {
-                            index = index.saturating_sub(self.account_deps.len());
-                            &self.message.account_keys[index]
-                            // &self.accounts[index] as &RefCell<AccountSharedData>,
-                        };
-                        (*is_signer, *is_writable, key, transmute_lifetime(*account))
+                        (
+                            *is_signer,
+                            *is_writable,
+                            &self.accounts[index].0,
+                            // &self.accounts[index] as &RefCell<AccountSharedData>
+                            transmute_lifetime(*account),
+                        )
                     })
             })
             .collect::<Option<Vec<_>>>()
@@ -391,10 +391,9 @@ impl<'a> InvokeContext for ThisInvokeContext<'a> {
     }
     fn verify_and_update(
         &mut self,
-        message: &Message,
         instruction: &CompiledInstruction,
-        accounts: &[Rc<RefCell<AccountSharedData>>],
-        caller_write_privileges: Option<&[bool]>,
+        accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
+        write_privileges: &[bool],
     ) -> Result<(), InstructionError> {
         let stack_frame = self
             .invoke_stack
@@ -402,16 +401,15 @@ impl<'a> InvokeContext for ThisInvokeContext<'a> {
             .ok_or(InstructionError::CallDepth)?;
         let logger = self.get_logger();
         MessageProcessor::verify_and_update(
-            message,
             instruction,
             &mut self.pre_accounts,
             accounts,
             &stack_frame.key,
             &self.rent,
-            caller_write_privileges,
+            write_privileges,
             &mut self.timings,
-            self.feature_set.is_active(&demote_sysvar_write_locks::id()),
             logger,
+            self.feature_set.is_active(&updated_verify_policy::id()),
         )
     }
     fn get_caller(&self) -> Result<&Pubkey, InstructionError> {
@@ -462,16 +460,7 @@ impl<'a> InvokeContext for ThisInvokeContext<'a> {
         self.feature_set.is_active(feature_id)
     }
     fn get_account(&self, pubkey: &Pubkey) -> Option<Rc<RefCell<AccountSharedData>>> {
-        if let Some(account) = self.pre_accounts.iter().find_map(|pre| {
-            if pre.key == *pubkey {
-                Some(pre.account.clone())
-            } else {
-                None
-            }
-        }) {
-            return Some(account);
-        }
-        self.account_deps.iter().find_map(|(key, account)| {
+        self.accounts.iter().find_map(|(key, account)| {
             if key == pubkey {
                 Some(account.clone())
             } else {
@@ -621,8 +610,7 @@ impl MessageProcessor {
         message: &'a Message,
         instruction: &'a CompiledInstruction,
         executable_accounts: &'a [(Pubkey, Rc<RefCell<AccountSharedData>>)],
-        accounts: &'a [Rc<RefCell<AccountSharedData>>],
-        demote_sysvar_write_locks: bool,
+        accounts: &'a [(Pubkey, Rc<RefCell<AccountSharedData>>)],
     ) -> Vec<(bool, bool, &'a Pubkey, &'a RefCell<AccountSharedData>)> {
         executable_accounts
             .iter()
@@ -631,9 +619,9 @@ impl MessageProcessor {
                 let index = *index as usize;
                 (
                     message.is_signer(index),
-                    message.is_writable(index, demote_sysvar_write_locks),
-                    &message.account_keys[index],
-                    &accounts[index] as &RefCell<AccountSharedData>,
+                    message.is_writable(index),
+                    &accounts[index].0,
+                    &accounts[index].1 as &RefCell<AccountSharedData>,
                 )
             }))
             .collect::<Vec<_>>()
@@ -768,7 +756,6 @@ impl MessageProcessor {
             accounts,
             keyed_account_indices_reordered,
             caller_write_privileges,
-            demote_sysvar_write_locks,
         ) = {
             let invoke_context = invoke_context.borrow();
 
@@ -793,7 +780,7 @@ impl MessageProcessor {
                 for keyed_account_index in keyed_account_indices {
                     let keyed_account = &keyed_accounts[*keyed_account_index];
                     if account_key == keyed_account.unsigned_key() {
-                        accounts.push(Rc::new(keyed_account.account.clone()));
+                        accounts.push((*account_key, Rc::new(keyed_account.account.clone())));
                         keyed_account_indices_reordered.push(*keyed_account_index);
                         continue 'root;
                     }
@@ -861,7 +848,6 @@ impl MessageProcessor {
                 accounts,
                 keyed_account_indices_reordered,
                 caller_write_privileges,
-                invoke_context.is_feature_active(&demote_sysvar_write_locks::id()),
             )
         };
 
@@ -879,16 +865,14 @@ impl MessageProcessor {
         {
             let invoke_context = invoke_context.borrow();
             let keyed_accounts = invoke_context.get_keyed_accounts()?;
-            for (src_keyed_account_index, (account, dst_keyed_account_index)) in accounts
+            for (src_keyed_account_index, ((_key, account), dst_keyed_account_index)) in accounts
                 .iter()
                 .zip(keyed_account_indices_reordered)
                 .enumerate()
             {
                 let dst_keyed_account = &keyed_accounts[dst_keyed_account_index];
                 let src_keyed_account = account.borrow();
-                if message.is_writable(src_keyed_account_index, demote_sysvar_write_locks)
-                    && !src_keyed_account.executable()
-                {
+                if message.is_writable(src_keyed_account_index) && !src_keyed_account.executable() {
                     if dst_keyed_account.data_len()? != src_keyed_account.data().len()
                         && dst_keyed_account.data_len()? != 0
                     {
@@ -921,7 +905,7 @@ impl MessageProcessor {
     pub fn process_cross_program_instruction(
         message: &Message,
         executable_accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
-        accounts: &[Rc<RefCell<AccountSharedData>>],
+        accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
         caller_write_privileges: &[bool],
         invoke_context: &mut dyn InvokeContext,
     ) -> Result<(), InstructionError> {
@@ -929,21 +913,11 @@ impl MessageProcessor {
             let program_id = instruction.program_id(&message.account_keys);
 
             // Verify the calling program hasn't misbehaved
-            invoke_context.verify_and_update(
-                message,
-                instruction,
-                accounts,
-                Some(caller_write_privileges),
-            )?;
+            invoke_context.verify_and_update(instruction, accounts, caller_write_privileges)?;
 
             // Construct keyed accounts
-            let keyed_accounts = Self::create_keyed_accounts(
-                message,
-                instruction,
-                executable_accounts,
-                accounts,
-                invoke_context.is_feature_active(&demote_sysvar_write_locks::id()),
-            );
+            let keyed_accounts =
+                Self::create_keyed_accounts(message, instruction, executable_accounts, accounts);
 
             // Invoke callee
             invoke_context.push(program_id, &keyed_accounts)?;
@@ -960,7 +934,10 @@ impl MessageProcessor {
             );
             if result.is_ok() {
                 // Verify the called program has not misbehaved
-                result = invoke_context.verify_and_update(message, instruction, accounts, None);
+                let write_privileges: Vec<bool> = (0..message.account_keys.len())
+                    .map(|i| message.is_writable(i))
+                    .collect();
+                result = invoke_context.verify_and_update(instruction, accounts, &write_privileges);
             }
 
             // Restore previous state
@@ -977,15 +954,17 @@ impl MessageProcessor {
     pub fn create_pre_accounts(
         message: &Message,
         instruction: &CompiledInstruction,
-        accounts: &[Rc<RefCell<AccountSharedData>>],
+        accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
     ) -> Vec<PreAccount> {
         let mut pre_accounts = Vec::with_capacity(instruction.accounts.len());
         {
             let mut work = |_unique_index: usize, account_index: usize| {
-                let key = &message.account_keys[account_index];
-                let account = accounts[account_index].borrow();
-                pre_accounts.push(PreAccount::new(key, &account));
-                Ok(())
+                if account_index < message.account_keys.len() && account_index < accounts.len() {
+                    let account = accounts[account_index].1.borrow();
+                    pre_accounts.push(PreAccount::new(&accounts[account_index].0, &account));
+                    return Ok(());
+                }
+                Err(InstructionError::MissingAccount)
             };
             let _ = instruction.visit_each_account(&mut work);
         }
@@ -1010,11 +989,11 @@ impl MessageProcessor {
         instruction: &CompiledInstruction,
         pre_accounts: &[PreAccount],
         executable_accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
-        accounts: &[Rc<RefCell<AccountSharedData>>],
+        accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
         rent: &Rent,
         timings: &mut ExecuteDetailsTimings,
-        demote_sysvar_write_locks: bool,
         logger: Rc<RefCell<dyn Logger>>,
+        updated_verify_policy: bool,
     ) -> Result<(), InstructionError> {
         // Verify all executable accounts have zero outstanding refs
         Self::verify_account_references(executable_accounts)?;
@@ -1027,18 +1006,20 @@ impl MessageProcessor {
                 {
                     // Verify account has no outstanding references
                     let _ = accounts[account_index]
+                        .1
                         .try_borrow_mut()
                         .map_err(|_| InstructionError::AccountBorrowOutstanding)?;
                 }
-                let account = accounts[account_index].borrow();
+                let account = accounts[account_index].1.borrow();
                 pre_accounts[unique_index]
                     .verify(
                         program_id,
-                        message.is_writable(account_index, demote_sysvar_write_locks),
+                        message.is_writable(account_index),
                         rent,
                         &account,
                         timings,
                         true,
+                        updated_verify_policy,
                     )
                     .map_err(|err| {
                         ic_logger_msg!(
@@ -1066,28 +1047,22 @@ impl MessageProcessor {
     /// Verify the results of a cross-program instruction
     #[allow(clippy::too_many_arguments)]
     fn verify_and_update(
-        message: &Message,
         instruction: &CompiledInstruction,
         pre_accounts: &mut [PreAccount],
-        accounts: &[Rc<RefCell<AccountSharedData>>],
+        accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
         program_id: &Pubkey,
         rent: &Rent,
-        caller_write_privileges: Option<&[bool]>,
+        write_privileges: &[bool],
         timings: &mut ExecuteDetailsTimings,
-        demote_sysvar_write_locks: bool,
         logger: Rc<RefCell<dyn Logger>>,
+        updated_verify_policy: bool,
     ) -> Result<(), InstructionError> {
         // Verify the per-account instruction results
         let (mut pre_sum, mut post_sum) = (0_u128, 0_u128);
         let mut work = |_unique_index: usize, account_index: usize| {
-            if account_index < message.account_keys.len() && account_index < accounts.len() {
-                let key = &message.account_keys[account_index];
-                let account = &accounts[account_index];
-                let is_writable = if let Some(caller_write_privileges) = caller_write_privileges {
-                    caller_write_privileges[account_index]
-                } else {
-                    message.is_writable(account_index, demote_sysvar_write_locks)
-                };
+            if account_index < write_privileges.len() && account_index < accounts.len() {
+                let (key, account) = &accounts[account_index];
+                let is_writable = write_privileges[account_index];
                 // Find the matching PreAccount
                 for pre_account in pre_accounts.iter_mut() {
                     if key == pre_account.key() {
@@ -1099,7 +1074,15 @@ impl MessageProcessor {
                         }
                         let account = account.borrow();
                         pre_account
-                            .verify(program_id, is_writable, rent, &account, timings, false)
+                            .verify(
+                                program_id,
+                                is_writable,
+                                rent,
+                                &account,
+                                timings,
+                                false,
+                                updated_verify_policy,
+                            )
                             .map_err(|err| {
                                 ic_logger_msg!(logger, "failed to verify account {}: {}", key, err);
                                 err
@@ -1135,8 +1118,7 @@ impl MessageProcessor {
         message: &Message,
         instruction: &CompiledInstruction,
         executable_accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
-        accounts: &[Rc<RefCell<AccountSharedData>>],
-        account_deps: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
+        accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
         rent_collector: &RentCollector,
         log_collector: Option<Rc<LogCollector>>,
         executors: Rc<RefCell<Executors>>,
@@ -1145,16 +1127,15 @@ impl MessageProcessor {
         feature_set: Arc<FeatureSet>,
         bpf_compute_budget: BpfComputeBudget,
         timings: &mut ExecuteDetailsTimings,
-        demote_sysvar_write_locks: bool,
         account_db: Arc<Accounts>,
         ancestors: &Ancestors,
     ) -> Result<(), InstructionError> {
         // Fixup the special instructions key if present
         // before the account pre-values are taken care of
         if feature_set.is_active(&instructions_sysvar_enabled::id()) {
-            for (i, key) in message.account_keys.iter().enumerate() {
-                if instructions::check_id(key) {
-                    let mut mut_account_ref = accounts[i].borrow_mut();
+            for (pubkey, accont) in accounts.iter().take(message.account_keys.len()) {
+                if instructions::check_id(pubkey) {
+                    let mut mut_account_ref = accont.borrow_mut();
                     instructions::store_current_index(
                         mut_account_ref.data_as_mut_slice(),
                         instruction_index as u16,
@@ -1165,6 +1146,16 @@ impl MessageProcessor {
         }
 
         let program_id = instruction.program_id(&message.account_keys);
+
+        let mut bpf_compute_budget = bpf_compute_budget;
+        if feature_set.is_active(&neon_evm_compute_budget::id())
+            && *program_id == crate::neon_evm_program::id()
+        {
+            // Bump the compute budget for neon_evm
+            bpf_compute_budget.max_units = 500_000;
+            bpf_compute_budget.heap_size = Some(256 * 1024);
+        }
+
         let mut invoke_context = ThisInvokeContext::new(
             program_id,
             rent_collector.rent,
@@ -1172,7 +1163,6 @@ impl MessageProcessor {
             instruction,
             executable_accounts,
             accounts,
-            account_deps,
             &self.programs,
             log_collector,
             bpf_compute_budget,
@@ -1191,8 +1181,8 @@ impl MessageProcessor {
             accounts,
             &rent_collector.rent,
             timings,
-            demote_sysvar_write_locks,
             invoke_context.get_logger(),
+            invoke_context.is_feature_active(&updated_verify_policy::id()),
         )?;
 
         timings.accumulate(&invoke_context.timings);
@@ -1209,8 +1199,7 @@ impl MessageProcessor {
         &self,
         message: &Message,
         loaders: &[Vec<(Pubkey, Rc<RefCell<AccountSharedData>>)>],
-        accounts: &[Rc<RefCell<AccountSharedData>>],
-        account_deps: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
+        accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
         rent_collector: &RentCollector,
         log_collector: Option<Rc<LogCollector>>,
         executors: Rc<RefCell<Executors>>,
@@ -1221,7 +1210,6 @@ impl MessageProcessor {
         account_db: Arc<Accounts>,
         ancestors: &Ancestors,
     ) -> Result<(), TransactionError> {
-        let demote_sysvar_write_locks = feature_set.is_active(&demote_sysvar_write_locks::id());
         for (instruction_index, instruction) in message.instructions.iter().enumerate() {
             let instruction_recorder = instruction_recorders
                 .as_ref()
@@ -1231,7 +1219,6 @@ impl MessageProcessor {
                 instruction,
                 &loaders[instruction_index],
                 accounts,
-                account_deps,
                 rent_collector,
                 log_collector.clone(),
                 executors.clone(),
@@ -1240,7 +1227,6 @@ impl MessageProcessor {
                 feature_set.clone(),
                 bpf_compute_budget,
                 timings,
-                demote_sysvar_write_locks,
                 account_db.clone(),
                 ancestors,
             )
@@ -1264,25 +1250,29 @@ mod tests {
     fn test_invoke_context() {
         const MAX_DEPTH: usize = 10;
         let mut invoke_stack = vec![];
-        let mut keys = vec![];
         let mut accounts = vec![];
         let mut metas = vec![];
         for i in 0..MAX_DEPTH {
             invoke_stack.push(solana_sdk::pubkey::new_rand());
-            keys.push(solana_sdk::pubkey::new_rand());
-            accounts.push(Rc::new(RefCell::new(AccountSharedData::new(
-                i as u64,
-                1,
-                &invoke_stack[i],
-            ))));
-            metas.push(AccountMeta::new(keys[i], false));
+            accounts.push((
+                solana_sdk::pubkey::new_rand(),
+                Rc::new(RefCell::new(AccountSharedData::new(
+                    i as u64,
+                    1,
+                    &invoke_stack[i],
+                ))),
+            ));
+            metas.push(AccountMeta::new(accounts[i].0, false));
         }
         for program_id in invoke_stack.iter() {
-            accounts.push(Rc::new(RefCell::new(AccountSharedData::new(
-                1,
-                1,
-                &solana_sdk::pubkey::Pubkey::default(),
-            ))));
+            accounts.push((
+                *program_id,
+                Rc::new(RefCell::new(AccountSharedData::new(
+                    1,
+                    1,
+                    &solana_sdk::pubkey::Pubkey::default(),
+                ))),
+            ));
             metas.push(AccountMeta::new(*program_id, false));
         }
 
@@ -1299,7 +1289,6 @@ mod tests {
             &[],
             &accounts,
             &[],
-            &[],
             None,
             BpfComputeBudget::default(),
             Rc::new(RefCell::new(Executors::default())),
@@ -1324,8 +1313,8 @@ mod tests {
         for owned_index in (1..depth_reached).rev() {
             let not_owned_index = owned_index - 1;
             let metas = vec![
-                AccountMeta::new(keys[not_owned_index], false),
-                AccountMeta::new(keys[owned_index], false),
+                AccountMeta::new(accounts[not_owned_index].0, false),
+                AccountMeta::new(accounts[owned_index].0, false),
             ];
             let message = Message::new(
                 &[Instruction::new_with_bytes(
@@ -1337,16 +1326,22 @@ mod tests {
             );
 
             // modify account owned by the program
-            accounts[owned_index].borrow_mut().data_as_mut_slice()[0] =
+            accounts[owned_index].1.borrow_mut().data_as_mut_slice()[0] =
                 (MAX_DEPTH + owned_index) as u8;
             let mut these_accounts = accounts[not_owned_index..owned_index + 1].to_vec();
-            these_accounts.push(Rc::new(RefCell::new(AccountSharedData::new(
-                1,
-                1,
-                &solana_sdk::pubkey::Pubkey::default(),
-            ))));
+            these_accounts.push((
+                message.account_keys[2],
+                Rc::new(RefCell::new(AccountSharedData::new(
+                    1,
+                    1,
+                    &solana_sdk::pubkey::Pubkey::default(),
+                ))),
+            ));
+            let write_privileges: Vec<bool> = (0..message.account_keys.len())
+                .map(|i| message.is_writable(i))
+                .collect();
             invoke_context
-                .verify_and_update(&message, &message.instructions[0], &these_accounts, None)
+                .verify_and_update(&message.instructions[0], &these_accounts, &write_privileges)
                 .unwrap();
             assert_eq!(
                 invoke_context.pre_accounts[owned_index]
@@ -1357,15 +1352,14 @@ mod tests {
             );
 
             // modify account not owned by the program
-            let data = accounts[not_owned_index].borrow_mut().data()[0];
-            accounts[not_owned_index].borrow_mut().data_as_mut_slice()[0] =
+            let data = accounts[not_owned_index].1.borrow_mut().data()[0];
+            accounts[not_owned_index].1.borrow_mut().data_as_mut_slice()[0] =
                 (MAX_DEPTH + not_owned_index) as u8;
             assert_eq!(
                 invoke_context.verify_and_update(
-                    &message,
                     &message.instructions[0],
                     &accounts[not_owned_index..owned_index + 1],
-                    None
+                    &write_privileges,
                 ),
                 Err(InstructionError::ExternalAccountDataModified)
             );
@@ -1376,7 +1370,7 @@ mod tests {
                     .data()[0],
                 data
             );
-            accounts[not_owned_index].borrow_mut().data_as_mut_slice()[0] = data;
+            accounts[not_owned_index].1.borrow_mut().data_as_mut_slice()[0] = data;
 
             invoke_context.pop();
         }
@@ -1485,6 +1479,7 @@ mod tests {
                 &self.post,
                 &mut ExecuteDetailsTimings::default(),
                 false,
+                true,
             )
         }
     }
@@ -1807,6 +1802,21 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_verify_account_changes_owner_executable() {
+        let alice_program_id = solana_sdk::pubkey::new_rand();
+        let bob_program_id = solana_sdk::pubkey::new_rand();
+
+        assert_eq!(
+            Change::new(&alice_program_id, &alice_program_id)
+                .owner(&bob_program_id)
+                .executable(false, true)
+                .verify(),
+            Err(InstructionError::ExecutableModified),
+            "Program should not be able to change owner and executable at the same time"
+        );
+    }
+
     #[test]
     fn test_process_message_readonly_handling() {
         #[derive(Serialize, Deserialize)]
@@ -1852,26 +1862,28 @@ mod tests {
         let mut message_processor = MessageProcessor::default();
         message_processor.add_program(mock_system_program_id, mock_system_process_instruction);
 
-        let mut accounts: Vec<Rc<RefCell<AccountSharedData>>> = Vec::new();
-        let account = AccountSharedData::new_ref(100, 1, &mock_system_program_id);
-        accounts.push(account);
-        let account = AccountSharedData::new_ref(0, 1, &mock_system_program_id);
-        accounts.push(account);
+        let accounts = vec![
+            (
+                solana_sdk::pubkey::new_rand(),
+                AccountSharedData::new_ref(100, 1, &mock_system_program_id),
+            ),
+            (
+                solana_sdk::pubkey::new_rand(),
+                AccountSharedData::new_ref(0, 1, &mock_system_program_id),
+            ),
+        ];
 
-        let mut loaders: Vec<Vec<(Pubkey, Rc<RefCell<AccountSharedData>>)>> = Vec::new();
         let account = Rc::new(RefCell::new(create_loadable_account_for_test(
             "mock_system_program",
         )));
-        loaders.push(vec![(mock_system_program_id, account)]);
+        let loaders = vec![vec![(mock_system_program_id, account)]];
 
         let executors = Rc::new(RefCell::new(Executors::default()));
         let ancestors = Ancestors::default();
 
-        let from_pubkey = solana_sdk::pubkey::new_rand();
-        let to_pubkey = solana_sdk::pubkey::new_rand();
         let account_metas = vec![
-            AccountMeta::new(from_pubkey, true),
-            AccountMeta::new_readonly(to_pubkey, false),
+            AccountMeta::new(accounts[0].0, true),
+            AccountMeta::new_readonly(accounts[1].0, false),
         ];
         let message = Message::new(
             &[Instruction::new_with_bincode(
@@ -1879,14 +1891,13 @@ mod tests {
                 &MockSystemInstruction::Correct,
                 account_metas.clone(),
             )],
-            Some(&from_pubkey),
+            Some(&accounts[0].0),
         );
 
         let result = message_processor.process_message(
             &message,
             &loaders,
             &accounts,
-            &[],
             &rent_collector,
             None,
             executors.clone(),
@@ -1898,8 +1909,8 @@ mod tests {
             &ancestors,
         );
         assert_eq!(result, Ok(()));
-        assert_eq!(accounts[0].borrow().lamports(), 100);
-        assert_eq!(accounts[1].borrow().lamports(), 0);
+        assert_eq!(accounts[0].1.borrow().lamports(), 100);
+        assert_eq!(accounts[1].1.borrow().lamports(), 0);
 
         let message = Message::new(
             &[Instruction::new_with_bincode(
@@ -1907,14 +1918,13 @@ mod tests {
                 &MockSystemInstruction::AttemptCredit { lamports: 50 },
                 account_metas.clone(),
             )],
-            Some(&from_pubkey),
+            Some(&accounts[0].0),
         );
 
         let result = message_processor.process_message(
             &message,
             &loaders,
             &accounts,
-            &[],
             &rent_collector,
             None,
             executors.clone(),
@@ -1939,14 +1949,13 @@ mod tests {
                 &MockSystemInstruction::AttemptDataChange { data: 50 },
                 account_metas,
             )],
-            Some(&from_pubkey),
+            Some(&accounts[0].0),
         );
 
         let result = message_processor.process_message(
             &message,
             &loaders,
             &accounts,
-            &[],
             &rent_collector,
             None,
             executors,
@@ -2032,28 +2041,29 @@ mod tests {
         let mut message_processor = MessageProcessor::default();
         message_processor.add_program(mock_program_id, mock_system_process_instruction);
 
-        let mut accounts: Vec<Rc<RefCell<AccountSharedData>>> = Vec::new();
-        let account = AccountSharedData::new_ref(100, 1, &mock_program_id);
-        accounts.push(account);
-        let account = AccountSharedData::new_ref(0, 1, &mock_program_id);
-        accounts.push(account);
+        let accounts = vec![
+            (
+                solana_sdk::pubkey::new_rand(),
+                AccountSharedData::new_ref(100, 1, &mock_program_id),
+            ),
+            (
+                solana_sdk::pubkey::new_rand(),
+                AccountSharedData::new_ref(0, 1, &mock_program_id),
+            ),
+        ];
 
-        let mut loaders: Vec<Vec<(Pubkey, Rc<RefCell<AccountSharedData>>)>> = Vec::new();
         let account = Rc::new(RefCell::new(create_loadable_account_for_test(
             "mock_system_program",
         )));
-        loaders.push(vec![(mock_program_id, account)]);
+        let loaders = vec![vec![(mock_program_id, account)]];
 
         let executors = Rc::new(RefCell::new(Executors::default()));
         let ancestors = Ancestors::default();
 
-        let from_pubkey = solana_sdk::pubkey::new_rand();
-        let to_pubkey = solana_sdk::pubkey::new_rand();
-        let dup_pubkey = from_pubkey;
         let account_metas = vec![
-            AccountMeta::new(from_pubkey, true),
-            AccountMeta::new(to_pubkey, false),
-            AccountMeta::new(dup_pubkey, false),
+            AccountMeta::new(accounts[0].0, true),
+            AccountMeta::new(accounts[1].0, false),
+            AccountMeta::new(accounts[0].0, false),
         ];
 
         // Try to borrow mut the same account
@@ -2063,13 +2073,12 @@ mod tests {
                 &MockSystemInstruction::BorrowFail,
                 account_metas.clone(),
             )],
-            Some(&from_pubkey),
+            Some(&accounts[0].0),
         );
         let result = message_processor.process_message(
             &message,
             &loaders,
             &accounts,
-            &[],
             &rent_collector,
             None,
             executors.clone(),
@@ -2095,13 +2104,12 @@ mod tests {
                 &MockSystemInstruction::MultiBorrowMut,
                 account_metas.clone(),
             )],
-            Some(&from_pubkey),
+            Some(&accounts[0].0),
         );
         let result = message_processor.process_message(
             &message,
             &loaders,
             &accounts,
-            &[],
             &rent_collector,
             None,
             executors.clone(),
@@ -2124,14 +2132,13 @@ mod tests {
                 },
                 account_metas,
             )],
-            Some(&from_pubkey),
+            Some(&accounts[0].0),
         );
         let ancestors = Ancestors::default();
         let result = message_processor.process_message(
             &message,
             &loaders,
             &accounts,
-            &[],
             &rent_collector,
             None,
             executors,
@@ -2143,9 +2150,9 @@ mod tests {
             &ancestors,
         );
         assert_eq!(result, Ok(()));
-        assert_eq!(accounts[0].borrow().lamports(), 80);
-        assert_eq!(accounts[1].borrow().lamports(), 20);
-        assert_eq!(accounts[0].borrow().data(), &vec![42]);
+        assert_eq!(accounts[0].1.borrow().lamports(), 80);
+        assert_eq!(accounts[1].1.borrow().lamports(), 20);
+        assert_eq!(accounts[0].1.borrow().data(), &vec![42]);
     }
 
     #[test]
@@ -2197,25 +2204,28 @@ mod tests {
             Rc::new(RefCell::new(program_account.clone())),
         )];
 
-        let owned_key = solana_sdk::pubkey::new_rand();
         let owned_account = AccountSharedData::new(42, 1, &callee_program_id);
-
-        let not_owned_key = solana_sdk::pubkey::new_rand();
         let not_owned_account = AccountSharedData::new(84, 1, &solana_sdk::pubkey::new_rand());
 
         #[allow(unused_mut)]
-        let mut accounts = vec![
-            Rc::new(RefCell::new(owned_account)),
-            Rc::new(RefCell::new(not_owned_account)),
-            Rc::new(RefCell::new(program_account)),
+        let accounts = vec![
+            (
+                solana_sdk::pubkey::new_rand(),
+                Rc::new(RefCell::new(owned_account)),
+            ),
+            (
+                solana_sdk::pubkey::new_rand(),
+                Rc::new(RefCell::new(not_owned_account)),
+            ),
+            (callee_program_id, Rc::new(RefCell::new(program_account))),
         ];
 
         let compiled_instruction = CompiledInstruction::new(2, &(), vec![0, 1, 2]);
         let programs: Vec<(_, ProcessInstructionWithContext)> =
             vec![(callee_program_id, mock_process_instruction)];
         let metas = vec![
-            AccountMeta::new(owned_key, false),
-            AccountMeta::new(not_owned_key, false),
+            AccountMeta::new(accounts[0].0, false),
+            AccountMeta::new(accounts[1].0, false),
         ];
 
         let instruction = Instruction::new_with_bincode(
@@ -2233,7 +2243,6 @@ mod tests {
             &compiled_instruction,
             &executable_accounts,
             &accounts,
-            &[],
             programs.as_slice(),
             None,
             BpfComputeBudget::default(),
@@ -2245,15 +2254,13 @@ mod tests {
         );
 
         // not owned account modified by the caller (before the invoke)
-        let demote_sysvar_write_locks =
-            invoke_context.is_feature_active(&demote_sysvar_write_locks::id());
         let caller_write_privileges = message
             .account_keys
             .iter()
             .enumerate()
-            .map(|(i, _)| message.is_writable(i, demote_sysvar_write_locks))
+            .map(|(i, _)| message.is_writable(i))
             .collect::<Vec<bool>>();
-        accounts[0].borrow_mut().data_as_mut_slice()[0] = 1;
+        accounts[0].1.borrow_mut().data_as_mut_slice()[0] = 1;
         assert_eq!(
             MessageProcessor::process_cross_program_instruction(
                 &message,
@@ -2264,7 +2271,7 @@ mod tests {
             ),
             Err(InstructionError::ExternalAccountDataModified)
         );
-        accounts[0].borrow_mut().data_as_mut_slice()[0] = 0;
+        accounts[0].1.borrow_mut().data_as_mut_slice()[0] = 0;
 
         let cases = vec![
             (MockInstruction::NoopSuccess, Ok(())),
@@ -2292,7 +2299,6 @@ mod tests {
                 &compiled_instruction,
                 &executable_accounts,
                 &accounts,
-                &[],
                 programs.as_slice(),
                 None,
                 BpfComputeBudget::default(),
@@ -2307,7 +2313,7 @@ mod tests {
                 .account_keys
                 .iter()
                 .enumerate()
-                .map(|(i, _)| message.is_writable(i, demote_sysvar_write_locks))
+                .map(|(i, _)| message.is_writable(i))
                 .collect::<Vec<bool>>();
             assert_eq!(
                 MessageProcessor::process_cross_program_instruction(
