@@ -60,10 +60,11 @@ impl RentCollector {
         &self,
         address: &Pubkey,
         account: &mut AccountSharedData,
+        rent_for_sysvars: bool,
     ) -> u64 {
         if account.executable() // executable accounts must be rent-exempt balance
             || account.rent_epoch() > self.epoch
-            || sysvar::check_id(account.owner())
+            || (!rent_for_sysvars && sysvar::check_id(account.owner()))
             || *address == incinerator::id()
         {
             0
@@ -115,10 +116,11 @@ impl RentCollector {
         &self,
         address: &Pubkey,
         account: &mut AccountSharedData,
+        rent_for_sysvars: bool,
     ) -> u64 {
         // initialize rent_epoch as created at this epoch
         account.set_rent_epoch(self.epoch);
-        self.collect_from_existing_account(address, account)
+        self.collect_from_existing_account(address, account, rent_for_sysvars)
     }
 }
 
@@ -146,15 +148,21 @@ mod tests {
         let rent_collector = RentCollector::default().clone_with_epoch(new_epoch);
 
         // collect rent on a newly-created account
-        let collected = rent_collector
-            .collect_from_created_account(&solana_sdk::pubkey::new_rand(), &mut created_account);
+        let collected = rent_collector.collect_from_created_account(
+            &solana_sdk::pubkey::new_rand(),
+            &mut created_account,
+            true,
+        );
         assert!(created_account.lamports() < old_lamports);
         assert_eq!(created_account.lamports() + collected, old_lamports);
         assert_ne!(created_account.rent_epoch(), old_epoch);
 
         // collect rent on a already-existing account
-        let collected = rent_collector
-            .collect_from_existing_account(&solana_sdk::pubkey::new_rand(), &mut existing_account);
+        let collected = rent_collector.collect_from_existing_account(
+            &solana_sdk::pubkey::new_rand(),
+            &mut existing_account,
+            true,
+        );
         assert!(existing_account.lamports() < old_lamports);
         assert_eq!(existing_account.lamports() + collected, old_lamports);
         assert_ne!(existing_account.rent_epoch(), old_epoch);
@@ -180,7 +188,7 @@ mod tests {
         let rent_collector = RentCollector::default().clone_with_epoch(epoch);
 
         // first mark account as being collected while being rent-exempt
-        collected = rent_collector.collect_from_existing_account(&pubkey, &mut account);
+        collected = rent_collector.collect_from_existing_account(&pubkey, &mut account, true);
         assert_eq!(account.lamports(), huge_lamports);
         assert_eq!(collected, 0);
 
@@ -188,8 +196,33 @@ mod tests {
         account.set_lamports(tiny_lamports);
 
         // ... and trigger another rent collection on the same epoch and check that rent is working
-        collected = rent_collector.collect_from_existing_account(&pubkey, &mut account);
+        collected = rent_collector.collect_from_existing_account(&pubkey, &mut account, true);
         assert_eq!(account.lamports(), tiny_lamports - collected);
         assert_ne!(collected, 0);
     }
+
+    #[test]
+    fn test_rent_exempt_sysvar() {
+        let tiny_lamports = 1;
+        let mut account = AccountSharedData::default();
+        account.set_owner(sysvar::id());
+        account.set_lamports(tiny_lamports);
+
+        let pubkey = solana_sdk::pubkey::new_rand();
+
+        assert_eq!(account.rent_epoch(), 0);
+
+        let epoch = 3;
+        let rent_collector = RentCollector::default().clone_with_epoch(epoch);
+
+        // old behavior: sysvars are special-cased
+        let collected = rent_collector.collect_from_existing_account(&pubkey, &mut account, false);
+        assert_eq!(account.lamports(), tiny_lamports);
+        assert_eq!(collected, 0);
+
+        // new behavior: sysvars are NOT special-cased
+        let collected = rent_collector.collect_from_existing_account(&pubkey, &mut account, true);
+        assert_eq!(account.lamports(), 0);
+        assert_eq!(collected, 1);
+    }
 }
